package org.example;

import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.ClientProperties;
import org.glassfish.jersey.client.RequestEntityProcessing;
import org.glassfish.jersey.client.http.Expect100ContinueFeature;
import org.glassfish.jersey.logging.LoggingFeature;
import org.glassfish.jersey.media.multipart.BodyPart;
import org.glassfish.jersey.media.multipart.FormDataBodyPart;
import org.glassfish.jersey.media.multipart.FormDataContentDisposition;
import org.glassfish.jersey.media.multipart.FormDataMultiPart;
import org.glassfish.jersey.netty.connector.NettyConnectorProvider;
import javax.ws.rs.client.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

public class TestMain {

    public static void main(String[] args) {
        enableLogging(Level.FINE);
        test();
    }

    public static void test(){
        ClientConfig defaultConfig = new ClientConfig();
        defaultConfig.property(LoggingFeature.LOGGING_FEATURE_VERBOSITY_CLIENT, LoggingFeature.Verbosity.PAYLOAD_ANY);

        //The issue can be produced only by using NettyConnectorProvider
        defaultConfig.connectorProvider(new NettyConnectorProvider());

        //with below two lines, enabled 100-continue feature
        defaultConfig.property(ClientProperties.REQUEST_ENTITY_PROCESSING, RequestEntityProcessing.CHUNKED);
        defaultConfig.register(Expect100ContinueFeature.basic());

        Client client = ClientBuilder.newClient(defaultConfig);
        WebTarget webTarget = client.target("http://127.0.0.1:3000");
        Invocation.Builder invocationBuilder = webTarget.request();
        invocationBuilder.header("Accept", "application/json");

        Response response = invocationBuilder.post(generateSimpleEntity());

        System.out.println("Response status = " + response.getStatus());
        System.out.println("Response body = " + response.readEntity(String.class));
    }

    private static Entity<String> generateSimpleEntity(){
        return Entity.entity("{\"message\":\"Hello from java client\"}", MediaType.APPLICATION_JSON_TYPE);
    }

    private static Entity<BodyPart> generateMultipartEntity(){
        FormDataMultiPart multipart = new FormDataMultiPart();
        try {
            InputStream inputStream = new ByteArrayInputStream("file content".getBytes());
            String fileName = "200462-TEST_OC_366771.json";
            FormDataBodyPart fdp = new FormDataBodyPart(fileName, inputStream, MediaType.APPLICATION_JSON_TYPE);
            FormDataContentDisposition cd = new FormDataContentDisposition(
                    "form-data; name=\""
                            + "file"
                            + "\"; filename=\""
                            + fileName + "\";"
            );
            fdp.setContentDisposition(cd);
            multipart.bodyPart(fdp);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Entity.entity(multipart, MediaType.MULTIPART_FORM_DATA);
    }

    private static void enableLogging(Level logLevel) {
        Logger rootLogger = Logger.getLogger("");
        rootLogger.setLevel(logLevel);
        Logger nettyLog = Logger.getLogger("io.netty");
        nettyLog.setLevel(logLevel);
        Handler[] handlers = rootLogger.getHandlers();
        for(Handler handler : handlers) {
            handler.setLevel(logLevel);
        }
    }
}
