const http = require('http');

// Create an HTTP server
const server = http.createServer((req, res) => {
    if (req.method === 'POST') {
        let body = '';

        req.on('data', chunk => {
            body += chunk;
        });

        req.on('end', () => {
            console.log('Server received body:', body);
            res.writeHead(200, { 'Content-Type': 'application/json' });
            res.end('{"message": "Request processed successfully."}');
        });
    } else {
        res.writeHead(405, { 'Content-Type': 'text/plain' });
        res.end('Only POST method is supported.\n');
    }
});

// Listen for 'checkContinue' event to handle `Expect: 100-continue`
server.on('checkContinue', (req, res) => {
    console.log('Server received Expect: 100-continue');
    res.writeContinue(); // Send HTTP/1.1 100 Continue

    let body = '';
    req.on('data', chunk => {
        body += chunk;
    });

    req.on('end', () => {
        console.log('Server received body after 100-continue:', body);
        res.writeHead(200, { 'Content-Type': 'application/json' });
        res.end('{"message": "Request processed successfully after 100-continue."}');
    });
});

// Start the server
const PORT = 3000;
server.listen(PORT, () => {
    console.log(`Server is listening on port ${PORT}`);
});

/**
 * curl request with 100-continue:
 * curl -v --location 'http://127.0.0.1:3000' --header 'Content-Type: application/json' --header 'expect: 100-continue' --data '{"msg": "hello from curl"}'
 *
 * curl request without 100-continue:
 * curl -v --location 'http://127.0.0.1:3000' --header 'Content-Type: application/json' --data '{"msg": "hello from curl"}'
 *
 */